//
// rustc thread.rs && ./thread
//
use std::sync::{Arc, Mutex};
use std::thread;
use std::time::Duration;

// 配列のサイズ
const G_BUF_SIZE: usize = 8;

// スレッド関数
fn sub_thread(shared_buf: Arc<Mutex<[i32; G_BUF_SIZE]>>) {
	let mut buf = shared_buf.lock().unwrap();

	for i in 0..G_BUF_SIZE {
		buf[i] = buf[i] * 2;
	}

	println!("buf in thread: {:?}", *buf);
}

// main関数
fn main() {
	// バッファの定義
	let buf = [0i32; G_BUF_SIZE];

    // スレッド間で共有する配列を作成する
    let shared_buf = Arc::new(Mutex::new(buf));

    // スレッドを作成する
    let shared_buf_clone = Arc::clone(&shared_buf);
    let thread_handle = thread::spawn(move || {
		sub_thread(shared_buf_clone);
    });

	//thread::sleep(Duration::from_millis(100));
	thread::sleep(Duration::from_millis(0));

    {
		let shared_buf_clone = Arc::clone(&shared_buf);
        let mut buf = shared_buf_clone.lock().unwrap();
        for i in 0..G_BUF_SIZE {
            buf[i] = buf[i] as i32 + 1;
        }
    } 

    // スレッドの終了を待つ
    thread_handle.join().unwrap();

    let final_buf = shared_buf.lock().unwrap();
    println!("buf: main {:?}", *final_buf);
}

